@@ -4,7 +4,9 @@
 fit best estimate of magnetometer offsets, diagonals, off-diagonals, cmot and scaling using WMM target
 '''
 
+from MAVProxy.modules.lib import wx_processguard
 from MAVProxy.modules.lib.wx_loader import wx
+
 import sys, time, os, math, copy
 
 from pymavlink import mavutil
@@ -12,6 +14,7 @@ from pymavlink import mavextra
 from pymavlink.rotmat import Vector3
 from pymavlink.rotmat import Matrix3
 from MAVProxy.modules.lib import grapher
+from MAVProxy.modules.lib.multiproc_util import MPDataLogChildTask
 
 import matplotlib
 import matplotlib.pyplot as pyplot
@@ -420,12 +423,73 @@ def magfit(mlog, timestamp_in_range):
 
     pyplot.show(block=False)
 
+class MagFit(MPDataLogChildTask):
+    '''A class used to launch the MagFitUI in a child process'''
+
+    def __init__(self, *args, **kwargs):
+        '''
+        Parameters
+        ----------
+        title : str
+            The title of the application
+        mlog : DFReader
+            A dataflash or telemetry log
+        xlimits: MAVExplorer.XLimits
+            An object capturing timestamp limits
+        '''
+
+        super(MagFit, self).__init__(*args, **kwargs)
+
+        # all attributes are implicitly passed to the child process 
+        self.title = kwargs['title']
+        self.xlimits = kwargs['xlimits']
+
+    # @override
+    def child_task(self):
+        '''Launch the MagFitUI'''
+
+        from MAVProxy.modules.lib import wx_processguard
+        from MAVProxy.modules.lib.wx_loader import wx
+
+        # create wx application
+        app = wx.App(False)
+        app.frame = MagFitUI(title=self.title,
+                             close_event=self.close_event,
+                             mlog=self.mlog,
+                             timestamp_in_range=self.xlimits.timestamp_in_range)
+
+        app.frame.SetDoubleBuffered(True)
+        app.frame.Show()
+        app.MainLoop()
 
 class MagFitUI(wx.Dialog):
-    def __init__(self, mlog, timestamp_in_range):
+    def __init__(self, title, close_event, mlog, timestamp_in_range):
+        super(MagFitUI, self).__init__(None, title=title, size=(600, 800))
+
+        # capture the close event, log and timestamp range function
+        self.close_event = close_event
         self.mlog = mlog
         self.timestamp_in_range = timestamp_in_range
-        self.closed = False
+
+        # events
+        self.timer = wx.Timer(self)
+        self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)
+        self.timer.Start(100)
+        self.Bind(wx.EVT_IDLE, self.OnIdle)
+
+        # initialise the panels etc.
+        self.init_ui()
+
+    def OnIdle(self, event):
+        time.sleep(0.05)
+
+    def OnTimer(self, event):
+        '''Periodically check if the close event has been received'''
+
+        if self.close_event.wait(0.001):
+            self.timer.Stop()
+            self.Destroy()
+            return
 
     def have_msg(self, msg):
         '''see if we have a given message name in the log'''
@@ -434,12 +498,13 @@ class MagFitUI(wx.Dialog):
             return False
         return self.mlog.counts[mid] > 0
 
-    def show(self):
+    def init_ui(self):
+        '''Initalise the UI elements'''
+
         if not hasattr(self.mlog, 'formats'):
             print("Must be DF log")
             return
-        self.app = wx.App(False)
-        wx.Dialog.__init__(self, None, -1, "MagFit", size=(600, 800))
+
         self.panel = wx.Panel(self)
         self.vbox = wx.BoxSizer(wx.VERTICAL)
         self.vbox.AddStretchSpacer()
@@ -506,15 +571,11 @@ class MagFitUI(wx.Dialog):
         self.EndRow()
         
         self.Center()
-        while not self.closed:
-            self.Show()
-            pyplot.pause(0.05)
-            time.sleep(0.05)
-            wx.Yield()
-        self.Close()
 
     def close(self):
-        self.closed = True
+        '''Set the close event'''
+
+        self.close_event.set()
 
     def StartRow(self, label=None):
         if self.row:
